@@ -23,6 +23,7 @@
 #include <bonobo/bonobo-ui-sync.h>
 #include <bonobo/bonobo-ui-sync-toolbar.h>
 #include <bonobo/bonobo-ui-preferences.h>
+#include <bonobo/bonobo-ui-private.h>
 
 #include <bonobo/bonobo-ui-toolbar.h>
 #include <bonobo/bonobo-ui-toolbar-button-item.h>
@@ -31,28 +32,22 @@
 #include <bonobo/bonobo-ui-toolbar-popup-item.h>
 #include <bonobo/bonobo-ui-toolbar-control-item.h>
 
-static BonoboUISyncClass *parent_class = NULL;
+static GObjectClass *parent_class = NULL;
 
 #define PARENT_TYPE bonobo_ui_sync_get_type ()
 
-static GtkWidget *
-cmd_get_toolbar_image (BonoboUINode     *node,
-		       BonoboUINode     *cmd_node)
+static GdkPixbuf *
+cmd_get_toolbar_image (GtkWidget    *widget,
+		       BonoboUINode *node,
+		       BonoboUINode *cmd_node)
 {
-	GtkWidget *image;
-	char      *type;
+	if (bonobo_ui_node_peek_attr (node, "pixtype"))
+		return bonobo_ui_util_xml_get_pixbuf (
+			widget, node, GTK_ICON_SIZE_LARGE_TOOLBAR);
 
-	if ((type = bonobo_ui_node_get_attr (node, "pixtype"))) {
-		image = bonobo_ui_util_xml_get_icon_widget (node, GTK_ICON_SIZE_SMALL_TOOLBAR);
-		bonobo_ui_node_free_string (type);
-		return image;
-	}
-
-	if ((type = bonobo_ui_node_get_attr (cmd_node, "pixtype"))) {
-		image = bonobo_ui_util_xml_get_icon_widget (cmd_node, GTK_ICON_SIZE_SMALL_TOOLBAR);
-		bonobo_ui_node_free_string (type);
-		return image;
-	}
+	if (bonobo_ui_node_peek_attr (cmd_node, "pixtype"))
+		return bonobo_ui_util_xml_get_pixbuf (
+			widget, cmd_node, GTK_ICON_SIZE_LARGE_TOOLBAR);
 
 	return NULL;
 }
@@ -97,7 +92,7 @@ impl_bonobo_ui_sync_toolbar_state (BonoboUISync     *sync,
 	char *min_width;
 	char *behavior;
 	char **behavior_array;
-	GtkWidget *image;
+	GdkPixbuf *image;
 	gboolean priority;
 
 	/* FIXME: to debug control problem */
@@ -128,13 +123,13 @@ impl_bonobo_ui_sync_toolbar_state (BonoboUISync     *sync,
 	bonobo_ui_toolbar_item_set_want_label (
 		BONOBO_UI_TOOLBAR_ITEM (widget), priority);
 
-	image = cmd_get_toolbar_image (node, cmd_node);
-
 	type  = bonobo_ui_engine_get_attr (node, cmd_node, "type");
 	label = bonobo_ui_engine_get_attr (node, cmd_node, "label");
 	
 	if (!type || !strcmp (type, "toggle")) {
 
+		image = cmd_get_toolbar_image (widget, node, cmd_node);
+
 		if (image) {
 			bonobo_ui_toolbar_button_item_set_image (
 				BONOBO_UI_TOOLBAR_BUTTON_ITEM (widget), image);
@@ -149,19 +144,27 @@ impl_bonobo_ui_sync_toolbar_state (BonoboUISync     *sync,
 	bonobo_ui_node_free_string (label);
 
 	if (bonobo_ui_node_has_name (node, "control")) {
-		char *txt;
+		const char *txt;
 		BonoboUIToolbarControlDisplay hdisp, vdisp;
+		gboolean sensitive;
 		
-		txt = bonobo_ui_node_get_attr (node, "hdisplay");
+		txt = bonobo_ui_node_peek_attr (node, "hdisplay");
 		hdisp = decode_control_disp (txt);
-		bonobo_ui_node_free_string (txt);
 
-		txt = bonobo_ui_node_get_attr (node, "vdisplay");
+		txt = bonobo_ui_node_peek_attr (node, "vdisplay");
 		vdisp = decode_control_disp (txt);
-		bonobo_ui_node_free_string (txt);
 
 		bonobo_ui_toolbar_control_item_set_display (
 			BONOBO_UI_TOOLBAR_CONTROL_ITEM (widget), hdisp, vdisp);
+
+		txt = bonobo_ui_node_peek_attr (node, "sensitive");
+		if (txt)
+			sensitive = atoi (txt);
+		else
+			sensitive = TRUE;
+
+		bonobo_ui_toolbar_control_item_set_sensitive (
+			BONOBO_UI_TOOLBAR_CONTROL_ITEM (widget), sensitive);
 	}
 
 	if ((min_width = bonobo_ui_engine_get_attr (node, cmd_node, "min_width"))) {
@@ -346,13 +349,13 @@ impl_bonobo_ui_sync_toolbar_build (BonoboUISync     *sync,
 	if (widget) {
 		/* FIXME: What about "id"s ! ? */
 		if ((verb = bonobo_ui_engine_get_attr (node, NULL, "verb"))) {
-			gtk_signal_connect (GTK_OBJECT (widget), "activate",
+			g_signal_connect (GTK_OBJECT (widget), "activate",
 					    (GtkSignalFunc) exec_verb_cb,
 					    sync->engine);
 			bonobo_ui_node_free_string (verb);
 		}
 		
-		gtk_signal_connect (GTK_OBJECT (widget), "state_altered",
+		g_signal_connect (GTK_OBJECT (widget), "state_altered",
 				    (GtkSignalFunc) win_item_emit_ui_event,
 				    sync->engine);
 	}
@@ -397,12 +400,11 @@ static GList *
 impl_bonobo_ui_sync_toolbar_get_widgets (BonoboUISync *sync,
 					 BonoboUINode *node)
 {
-	char          *dockname;
+	const char     *dockname;
 	BonoboDockItem *item;
 
-	dockname = bonobo_ui_node_get_attr (node, "name");
+	dockname = bonobo_ui_node_peek_attr (node, "name");
 	item = get_dock_item (BONOBO_UI_SYNC_TOOLBAR (sync), dockname);
-	bonobo_ui_node_free_string (dockname);
 
 	if (!item) {
 		g_warning ("Serious internal error building toolbar");
@@ -429,18 +431,22 @@ impl_bonobo_ui_sync_toolbar_state_update (BonoboUISync *sync,
 			g_warning ("TESTME: strange, setting "
 				   "state '%s' on weird object '%s'",
 				   new_state,
-				   GTK_CLASS_NAME (GTK_OBJECT_GET_CLASS (widget)));
+				   g_type_name_from_instance (
+					   (GTypeInstance *) widget));
 	}
 }
 
 static void
-impl_finalize (GObject *object)
+impl_dispose (GObject *object)
 {
-	BonoboUISyncToolbar *sync;
+	BonoboUISyncToolbar *sync = (BonoboUISyncToolbar *) object;
 
-	sync = BONOBO_UI_SYNC_TOOLBAR (object);
+	if (sync->dock) {
+		g_object_unref (sync->dock);
+		sync->dock = NULL;
+	}
 
-	G_OBJECT_CLASS (parent_class)->finalize (object);
+	parent_class->dispose (object);
 }
 
 static gboolean
@@ -464,36 +470,32 @@ parse_look (const char *look)
 			return BONOBO_UI_TOOLBAR_STYLE_PRIORITY_TEXT;	
 	}
 
-	return bonobo_ui_preferences_get_toolbar_labels ()
-		? BONOBO_UI_TOOLBAR_STYLE_ICONS_AND_TEXT
-		: BONOBO_UI_TOOLBAR_STYLE_ICONS_ONLY;
+	return bonobo_ui_preferences_get_toolbar_style ();
 }
 
 BonoboUIToolbarStyle
 bonobo_ui_sync_toolbar_get_look (BonoboUIEngine *engine,
 				 BonoboUINode   *node)
 {
-	char      *txt;
+	const char *txt;
 	BonoboUIToolbarStyle look;
 
-	if ((txt = bonobo_ui_node_get_attr (node, "look")))
+	if ((txt = bonobo_ui_node_peek_attr (node, "look")))
 		look = parse_look (txt);
 
 	else {
-		GtkWidget           *widget;
+		GtkWidget *widget;
 
 		widget = bonobo_ui_engine_node_get_widget (engine, node);
 
 		if (!widget || !BONOBO_IS_UI_TOOLBAR (widget) ||
 		    bonobo_ui_toolbar_get_orientation (BONOBO_UI_TOOLBAR (widget)) ==
 		    GTK_ORIENTATION_HORIZONTAL) {
-			txt = bonobo_ui_node_get_attr (node, "hlook");
+			txt = bonobo_ui_node_peek_attr (node, "hlook");
 			look = parse_look (txt);
-			bonobo_ui_node_free_string (txt);
 		} else {
-			txt = bonobo_ui_node_get_attr (node, "vlook");
+			txt = bonobo_ui_node_peek_attr (node, "vlook");
 			look = parse_look (txt);
-			bonobo_ui_node_free_string (txt);
 		}
 	}		
 	
@@ -505,20 +507,19 @@ do_config_popup (BonoboUIEngineConfig *config,
 		 BonoboUINode         *config_node,
 		 BonoboUIEngine       *popup_engine)
 {
-	char *txt;
+	char *ret;
 	gboolean tip;
+	const char *txt;
 	BonoboUIToolbarStyle style;
 	
 	tip = TRUE;
-	if ((txt = bonobo_ui_node_get_attr (config_node, "tips"))) {
+	if ((txt = bonobo_ui_node_peek_attr (config_node, "tips")))
 		tip = atoi (txt);
-		bonobo_ui_node_free_string (txt);
-	}
 
 	style = bonobo_ui_sync_toolbar_get_look (bonobo_ui_engine_config_get_engine (config),
 						 config_node);
 
-	txt = g_strdup_printf (
+	ret = g_strdup_printf (
 		"<Root>"
 		"<commands>"
 		"<cmd name=\"LookBoth\" state=\"%d\"/>"
@@ -551,7 +552,7 @@ do_config_popup (BonoboUIEngineConfig *config,
 		_("_Hide toolbar"), _("Customi_ze"),
 		_("Customize the toolbar"));
 
-	return txt;
+	return ret;
 }
 
 static void
@@ -561,13 +562,13 @@ config_verb_fn (BonoboUIEngineConfig *config,
 		BonoboUIEngine       *popup_engine,
 		BonoboUINode         *popup_node)
 {
-	char *verb;
+	const char *verb;
 	gboolean changed = TRUE;
 
-	if ((verb = bonobo_ui_node_get_attr (popup_node, "verb"))) {
-		char *set;
+	if ((verb = bonobo_ui_node_peek_attr (popup_node, "verb"))) {
+		const char *set;
 
-		set = bonobo_ui_node_get_attr (popup_node, "set");
+		set = bonobo_ui_node_peek_attr (popup_node, "set");
 
 		if (!strcmp (verb, "Hide"))
 			bonobo_ui_engine_config_add (
@@ -594,9 +595,6 @@ config_verb_fn (BonoboUIEngineConfig *config,
 
 		} else
 			g_warning ("Unknown verb '%s'", verb);
-
-		bonobo_ui_node_free_string (verb);
-		bonobo_ui_node_free_string (set);
 	}
 
 
@@ -611,8 +609,8 @@ create_dockitem (BonoboUISyncToolbar *sync,
 {
 	BonoboDockItem *item;
 	BonoboDockItemBehavior beh = 0;
-	char *prop;
-	char **behavior_array;
+	const char *prop;
+	char      **behavior_array;
 	gboolean force_detachable = FALSE;
 	BonoboDockPlacement placement = BONOBO_DOCK_TOP;
 	gint band_num = 1;
@@ -622,15 +620,13 @@ create_dockitem (BonoboUISyncToolbar *sync,
 	gboolean can_config = TRUE;
 	BonoboUIToolbar *toolbar;
 
-	if ((prop = bonobo_ui_node_get_attr (node, "behavior"))) {
+	if ((prop = bonobo_ui_node_peek_attr (node, "behavior"))) {
 		if (!strcmp (prop, "detachable"))
 			force_detachable = TRUE;
-		bonobo_ui_node_free_string (prop);
 	}
 
-	if ((prop = bonobo_ui_node_get_attr (node, "behavior"))) {
+	if ((prop = bonobo_ui_node_peek_attr (node, "behavior"))) {
 		behavior_array = g_strsplit (prop, ",", -1);
-		bonobo_ui_node_free_string (prop);
 	
 		if (string_array_contains (behavior_array, "detachable"))
 			force_detachable = TRUE;
@@ -656,14 +652,10 @@ create_dockitem (BonoboUISyncToolbar *sync,
 	item = BONOBO_DOCK_ITEM (bonobo_dock_item_new (
 		dockname, beh));
 
-	if (bonobo_ui_preferences_get_toolbar_relief ())
-		bonobo_dock_item_set_shadow_type (item, GTK_SHADOW_OUT);
-	else
-		bonobo_dock_item_set_shadow_type (item, GTK_SHADOW_NONE);
-
+	bonobo_dock_item_set_shadow_type (item, GTK_SHADOW_OUT);
 	gtk_container_set_border_width (GTK_CONTAINER (item), 2);
 
-	if ((prop = bonobo_ui_node_get_attr (node, "placement"))) {
+	if ((prop = bonobo_ui_node_peek_attr (node, "placement"))) {
 		if (!strcmp (prop, "top"))
 			placement = BONOBO_DOCK_TOP;
 		else if (!strcmp (prop, "right"))
@@ -674,32 +666,23 @@ create_dockitem (BonoboUISyncToolbar *sync,
 			placement = BONOBO_DOCK_LEFT;
 		else if (!strcmp (prop, "floating"))
 			placement = BONOBO_DOCK_FLOATING;
-		bonobo_ui_node_free_string (prop);
 	}
 
-	if ((prop = bonobo_ui_node_get_attr (node, "band_num"))) {
+	if ((prop = bonobo_ui_node_peek_attr (node, "band_num")))
 		band_num = atoi (prop);
-		bonobo_ui_node_free_string (prop);
-	}
 
-	if ((prop = bonobo_ui_node_get_attr (node, "position"))) {
+	if ((prop = bonobo_ui_node_peek_attr (node, "position")))
 		position = atoi (prop);
-		bonobo_ui_node_free_string (prop);
-	}
 
-	if ((prop = bonobo_ui_node_get_attr (node, "offset"))) {
+	if ((prop = bonobo_ui_node_peek_attr (node, "offset")))
 		offset = atoi (prop);
-		bonobo_ui_node_free_string (prop);
-	}
 
-	if ((prop = bonobo_ui_node_get_attr (node, "in_new_band"))) {
+	if ((prop = bonobo_ui_node_peek_attr (node, "in_new_band")))
 		in_new_band = atoi (prop);
-		bonobo_ui_node_free_string (prop);
-	}	
 
 	bonobo_dock_add_item (sync->dock, item,
-			     placement, band_num,
-			     position, offset, in_new_band);
+			      placement, band_num,
+			      position, offset, in_new_band);
 
 		
 	toolbar = BONOBO_UI_TOOLBAR (bonobo_ui_toolbar_new ());
@@ -708,10 +691,9 @@ create_dockitem (BonoboUISyncToolbar *sync,
 			   GTK_WIDGET (toolbar));
 	gtk_widget_show (GTK_WIDGET (toolbar));
 
-	if ((prop = bonobo_ui_node_get_attr (node, "config"))) {
+	if ((prop = bonobo_ui_node_peek_attr (node, "config")))
 		can_config = atoi (prop);
-		bonobo_ui_node_free_string (prop);
-	} else
+	else
 		can_config = TRUE;
 
 	if (can_config) {
@@ -736,9 +718,9 @@ static void
 impl_bonobo_ui_sync_toolbar_remove_root (BonoboUISync *sync,
 					 BonoboUINode *node)
 {
-	char *name = bonobo_ui_node_get_attr (node, "name");
+	const char *name;
 
-	if (name) {
+	if ((name = bonobo_ui_node_get_attr (node, "name"))) {
 		BonoboDockItem *item;
 
 		item = get_dock_item (BONOBO_UI_SYNC_TOOLBAR (sync), name);
@@ -746,21 +728,21 @@ impl_bonobo_ui_sync_toolbar_remove_root (BonoboUISync *sync,
 		if (item)
 			gtk_widget_destroy (GTK_WIDGET (item));
 	}
-
-	bonobo_ui_node_free_string (name);
 }
 
 static void
 impl_bonobo_ui_sync_toolbar_update_root (BonoboUISync *sync,
 					 BonoboUINode *node)
 {
-	char          *txt;
-	char          *dockname = bonobo_ui_node_get_attr (node, "name");
-	gboolean       tooltips;
+	const char *txt;
+	const char *dockname;
+	gboolean    tooltips;
 	BonoboDockItem *item;
 	BonoboUIToolbar *toolbar;
 	BonoboUIToolbarStyle look;
 
+	dockname = bonobo_ui_node_peek_attr (node, "name");
+
 	g_return_if_fail (dockname != NULL);
 
 	item = get_dock_item (BONOBO_UI_SYNC_TOOLBAR (sync), dockname);
@@ -773,21 +755,18 @@ impl_bonobo_ui_sync_toolbar_update_root (BonoboUISync *sync,
 
 	bonobo_ui_engine_stamp_root (sync->engine, node, GTK_WIDGET (toolbar));
 
-	if ((txt = bonobo_ui_node_get_attr (node, "look"))) {
+	if ((txt = bonobo_ui_node_peek_attr (node, "look"))) {
 		look = parse_look (txt);
 		bonobo_ui_toolbar_set_hv_styles (toolbar, look, look);
-		bonobo_ui_node_free_string (txt);
 
 	} else {
 		BonoboUIToolbarStyle vlook, hlook;
 
-		txt = bonobo_ui_node_get_attr (node, "hlook");
+		txt = bonobo_ui_node_peek_attr (node, "hlook");
 		hlook = parse_look (txt);
-		bonobo_ui_node_free_string (txt);
 
-		txt = bonobo_ui_node_get_attr (node, "vlook");
+		txt = bonobo_ui_node_peek_attr (node, "vlook");
 		vlook = parse_look (txt);
-		bonobo_ui_node_free_string (txt);
 
 		bonobo_ui_toolbar_set_hv_styles (toolbar, hlook, vlook);
 	}		
@@ -809,11 +788,11 @@ impl_bonobo_ui_sync_toolbar_update_root (BonoboUISync *sync,
 	}
 #endif
 
-	tooltips = TRUE;
-	if ((txt = bonobo_ui_node_get_attr (node, "tips"))) {
+	if ((txt = bonobo_ui_node_peek_attr (node, "tips")))
 		tooltips = atoi (txt);
-		bonobo_ui_node_free_string (txt);
-	}
+	else
+		tooltips = TRUE;
+
 	bonobo_ui_toolbar_show_tooltips (toolbar, tooltips);
 
        /*
@@ -824,8 +803,6 @@ impl_bonobo_ui_sync_toolbar_update_root (BonoboUISync *sync,
 	if (bonobo_ui_sync_do_show_hide (sync, node, NULL, GTK_WIDGET (item)))
 		gtk_widget_queue_resize (GTK_WIDGET (
 			BONOBO_UI_SYNC_TOOLBAR (sync)->dock));
-
-	bonobo_ui_node_free_string (dockname);
 }
 
 static gboolean
@@ -843,7 +820,7 @@ class_init (BonoboUISyncClass *sync_class)
 	parent_class = g_type_class_peek_parent (sync_class);
 
 	object_class = G_OBJECT_CLASS (sync_class);
-	object_class->finalize = impl_finalize;
+	object_class->dispose = impl_dispose;
 
 	sync_class->sync_state = impl_bonobo_ui_sync_toolbar_state;
 	sync_class->build      = impl_bonobo_ui_sync_toolbar_build;
@@ -893,7 +870,7 @@ bonobo_ui_sync_toolbar_new (BonoboUIEngine  *engine,
 
 	sync = g_object_new (BONOBO_TYPE_UI_SYNC_TOOLBAR, NULL);
 
-	sync->dock = dock;
+	sync->dock = g_object_ref (dock);
 
 	return bonobo_ui_sync_construct (
 		BONOBO_UI_SYNC (sync), engine, FALSE, TRUE);
