@@ -7,7 +7,6 @@
  * Copyright 2000, 2001 Ximian, Inc.
  */
 #include <config.h>
-#include <ctype.h>
 #include <string.h>
 
 #include <gtk/gtkstock.h>
@@ -19,7 +18,7 @@
 #include <bonobo/bonobo-ui-xml.h>
 #include <bonobo/bonobo-ui-util.h>
 #include <bonobo/bonobo-i18n.h>
-#include <bonobo/bonobo-ui-node-private.h>
+#include <bonobo/bonobo-ui-private.h>
 
 static gchar *find_pixmap_in_path (const gchar *filename);
 
@@ -180,7 +179,7 @@ bonobo_ui_util_xml_to_pixbuf (const char *xml)
 
 	g_return_val_if_fail (xml != NULL, NULL);
 
-	while (*xml && isspace ((unsigned char) (*xml)))
+	while (*xml && g_ascii_isspace (*xml))
 		xml++;
 
 	length = strlen (xml);
@@ -236,12 +235,40 @@ find_pixmap_in_path (const gchar *filename)
 	return file;
 }
 
+/* FIXME: could cut down allocation here a bit */
 static char *
 lookup_stock_compat (const char *id)
 {
 	GtkStockItem item;
-	char *lower = g_ascii_strdown (id, -1);
-	char *new_id = g_strconcat ("gtk-", lower, NULL);
+	char *lower, *new_id;
+	static GHashTable *compat_hash = NULL;
+
+	if (!compat_hash) {
+		static const char *mapping[][2] = {
+			{ "Save As",        "gtk-save-as" },
+			{ "Trash",          "gtk-delete" },
+			{ "Revert",         "gtk-revert-to-saved" },
+			{ "Exit",           "gtk-quit" },
+			{ "Search",         "gtk-find" },
+			{ "Search/Replace", "gtk-find-and-replace" },
+			{ "Timer Stopped",  "gnome-stock-timer-stop" },
+			{ "Scores",         "gnome-stock-scores" },
+			{ "About",          "gnome-stock-about" },
+			{ NULL, NULL }
+		};
+		int i;
+		compat_hash = g_hash_table_new (g_str_hash, g_str_equal);
+		for (i = 0; mapping [i][0]; i++)
+			g_hash_table_insert (compat_hash,
+					     (gpointer) mapping [i] [0],
+					     (gpointer) mapping [i] [1]);
+	}
+
+	if ((new_id = g_hash_table_lookup (compat_hash, id)))
+		return g_strdup (new_id);
+
+	lower = g_ascii_strdown (id, -1);
+	new_id = g_strconcat ("gtk-", lower, NULL);
 
 	if (gtk_stock_lookup (new_id, &item)) {
 		g_free (lower);
@@ -264,6 +291,84 @@ lookup_stock_compat (const char *id)
 	return NULL;
 }
 
+GdkPixbuf *
+bonobo_ui_util_xml_get_pixbuf (GtkWidget    *widget,
+			       BonoboUINode *node,
+			       GtkIconSize   icon_size)
+{
+	GdkPixbuf  *pixbuf = NULL;
+	char       *key;
+	const char *type, *text;
+	/* FIXME: this cache needs to be invalidated on theme
+	 * changes */
+	static GHashTable *pixbuf_cache = NULL;
+
+	g_return_val_if_fail (node != NULL, NULL);
+
+	if (!(type = bonobo_ui_node_peek_attr (node, "pixtype")))
+		return NULL;
+
+	if (!(text = bonobo_ui_node_peek_attr (node, "pixname")))
+		return NULL;
+
+	key = g_strdup_printf ("%s:%s:%d:%d", type, text,
+			       icon_size,
+			       gtk_widget_get_direction (widget));
+
+	if (!pixbuf_cache)
+		pixbuf_cache = g_hash_table_new (g_str_hash, g_str_equal);
+
+	else if ((pixbuf = g_hash_table_lookup (pixbuf_cache, key))) {
+		g_free (key);
+		g_object_ref (pixbuf);
+		return pixbuf;
+	}
+
+	if (!strcmp (type, "stock")) {
+		GtkStockItem item;
+
+		if (gtk_stock_lookup (text, &item))
+			pixbuf = gtk_widget_render_icon (
+				widget, text, icon_size, "bonobo-ui");
+		else {
+			char *mapped;
+
+			if ((mapped = lookup_stock_compat (text))) {
+				
+				pixbuf = gtk_widget_render_icon (
+					widget, mapped, icon_size, "bonobo-ui");
+
+				g_free (mapped);
+			} else
+				g_warning ("Unknown stock icon '%s', stock names all changed in Gtk+ 2.0", text);
+		}
+
+	} else if (!strcmp (type, "filename")) {
+		char *name = find_pixmap_in_path (text);
+
+		if ((name == NULL) || !g_file_test (name, G_FILE_TEST_EXISTS))
+			g_warning ("Could not find GNOME pixmap file %s", text);
+		else
+			pixbuf = gdk_pixbuf_new_from_file (name, NULL);
+
+		g_free (name);
+
+	} else if (!strcmp (type, "pixbuf"))
+		pixbuf = bonobo_ui_util_xml_to_pixbuf (text);
+
+	else
+		g_warning ("Unknown icon_pixbuf type '%s'", type);
+
+	if (pixbuf) {
+		g_object_ref (pixbuf);
+		g_hash_table_insert (pixbuf_cache, key, pixbuf);
+	} else
+		g_free (key);
+
+
+	return pixbuf;
+}
+
 /**
  * bonobo_ui_util_xml_get_icon_widget:
  * @node: the node
@@ -271,24 +376,22 @@ lookup_stock_compat (const char *id)
  * 
  * This function extracts a pixbuf from the node and returns a GtkWidget
  * containing a display of the pixbuf.
- * 
+ *
  * Return value: the widget.
  **/
 GtkWidget *
 bonobo_ui_util_xml_get_icon_widget (BonoboUINode *node, GtkIconSize icon_size)
 {
-	char      *type, *text;
-	GtkWidget *image = NULL;
+	const char *type, *text;
+	GtkWidget  *image = NULL;
 
 	g_return_val_if_fail (node != NULL, NULL);
 
-	if (!(type = bonobo_ui_node_get_attr (node, "pixtype")))
+	if (!(type = bonobo_ui_node_peek_attr (node, "pixtype")))
 		return NULL;
 
-	if (!(text = bonobo_ui_node_get_attr (node, "pixname"))) {
-		bonobo_ui_node_free_string (type);
+	if (!(text = bonobo_ui_node_peek_attr (node, "pixname")))
 		return NULL;
-	}
 
 	if (!text)
 		return NULL;
@@ -331,15 +434,19 @@ bonobo_ui_util_xml_get_icon_widget (BonoboUINode *node, GtkIconSize icon_size)
 	} else
 		g_warning ("Unknown icon_pixbuf type '%s'", type);
 
-	bonobo_ui_node_free_string (text);
-	bonobo_ui_node_free_string (type);
-
 	if (image)
 		gtk_widget_show (image);
 
 	return image;
 }
 
+void
+bonobo_ui_image_set_pixbuf (GtkImage *image, GdkPixbuf *pixbuf)
+{
+	if (gtk_image_get_pixbuf (image) != pixbuf)
+		gtk_image_set_from_pixbuf (image, pixbuf);
+}
+
 /**
  * bonobo_ui_util_xml_set_pixbuf:
  * @node: the node
@@ -477,12 +584,12 @@ bonobo_ui_util_build_help_menu (BonoboUIComponent *listener,
 		BonoboUINode *node;
 
 		/* Format of lines is "help_file_name whitespace* menu_title" */
-		for (s = buf; *s && !isspace (*s); s++)
+		for (s = buf; *s && !g_ascii_isspace (*s); s++)
 			;
 
 		*s++ = '\0';
 
-		for (; *s && isspace (*s); s++)
+		for (; *s && g_ascii_isspace (*s); s++)
 			;
 
 		if (s [strlen (s) - 1] == '\n')
@@ -505,7 +612,7 @@ bonobo_ui_util_build_help_menu (BonoboUIComponent *listener,
 		bonobo_ui_component_add_verb (listener, id,
 					      bonobo_help_display_cb, entry);
 
-		gtk_signal_connect (GTK_OBJECT (listener), "destroy",
+		g_signal_connect (GTK_OBJECT (listener), "destroy",
 				    (GtkSignalFunc) free_help_menu_entry, 
 				    entry);
 		g_free (id);
@@ -625,12 +732,10 @@ bonobo_ui_util_fixup_help (BonoboUIComponent *component,
 		return;
 
 	if (bonobo_ui_node_has_name (node, "placeholder")) {
-		char *txt;
+		const char *txt;
 
-		if ((txt = bonobo_ui_node_get_attr (node, "name"))) {
+		if ((txt = bonobo_ui_node_peek_attr (node, "name")))
 			build_here = !strcmp (txt, "BuiltMenuItems");
-			bonobo_ui_node_free_string (txt);
-		}
 	}
 
 	if (build_here) {
@@ -655,18 +760,16 @@ bonobo_ui_util_fixup_icons (BonoboUINode *node)
 {
 	BonoboUINode *l;
 	gboolean fixup_here = FALSE;
-	char *txt;
+	const char *txt;
 
 	if (!node)
 		return;
 
-	if ((txt = bonobo_ui_node_get_attr (node, "pixtype"))) {
+	if ((txt = bonobo_ui_node_peek_attr (node, "pixtype")))
 		fixup_here = !strcmp (txt, "filename");
-		bonobo_ui_node_free_string (txt);
-	}
 
 	if (fixup_here &&
-	    ((txt = bonobo_ui_node_get_attr (node, "pixname")))) {
+	    ((txt = bonobo_ui_node_peek_attr (node, "pixname")))) {
 		GdkPixbuf *pixbuf = NULL;
 
 		if (g_path_is_absolute (txt))
@@ -687,8 +790,6 @@ bonobo_ui_util_fixup_icons (BonoboUINode *node)
 			bonobo_ui_node_set_attr (node, "pixname", xml);
 			g_free (xml);
 		}
-
-		bonobo_ui_node_free_string (txt);
 	}
 
 	for (l = bonobo_ui_node_children (node); l; l = bonobo_ui_node_next (l))
@@ -963,7 +1064,7 @@ is_release (const gchar *string)
  * problems.
  **/
 void
-bonobo_ui_util_accel_parse (char              *accelerator,
+bonobo_ui_util_accel_parse (const char        *accelerator,
 			    guint             *accelerator_key,
 			    GdkModifierType   *accelerator_mods)
 {
@@ -1195,4 +1296,3 @@ bonobo_ui_util_set_pixbuf (BonoboUIComponent *component,
 
 	g_free (parent_path);
 }
-
