@@ -10,6 +10,7 @@
 #include <string.h>
 #include <stdlib.h>
 #include <libbonoboui.h>
+#include <gconf/gconf-client.h>
 #include <gdk/gdkkeysyms.h>
 
 #include <bonobo/bonobo-ui-xml.h>
@@ -33,9 +34,7 @@ dump_prefs (void)
 
 	fprintf (stderr, "Toolbar:\n");
 
-	PRINT_PREF ("labels", toolbar_labels);
 	PRINT_PREF ("detachable", toolbar_detachable);
-	PRINT_PREF ("relief", toolbar_relief);
 
 	fprintf (stderr, "Menus:\n");
 
@@ -45,7 +44,6 @@ dump_prefs (void)
 	fprintf (stderr, "Menubar:\n");
 
 	PRINT_PREF ("detachable", menubar_detachable);
-	PRINT_PREF ("relief", menubar_relief);
 }
 
 #undef PRINT_PREF
@@ -225,7 +223,7 @@ main (int argc, char **argv)
 		"</submenu>\n";
 	char simplec [] =
 		"<submenu name=\"File\" _label=\"_FileC\" _tip=\"what!\">\n"
-		"    <placeholder name=\"Nice\" delimit=\"top\" hidden=\"1\">\n"
+		"    <placeholder name=\"Nice\" delimit=\"top\" hidden=\"0\">\n"
 		"	<menuitem name=\"fooa\" _label=\"_FooA\" type=\"radio\" group=\"foogroup\" _tip=\"Radio1\"/>\n"
 		"	<menuitem name=\"foob\" _label=\"_FooB\" type=\"radio\" group=\"foogroup\" _tip=\"kippers\"/>\n"
 		"	<menuitem name=\"wibble\" verb=\"ThisForcesAnError\" _label=\"_Baa\""
@@ -260,6 +258,9 @@ main (int argc, char **argv)
 		"	<control name=\"Progress\"/>\n"
 		"</status>";
 
+	ev = &real_ev;
+	CORBA_exception_init (ev);
+
 	free (malloc (8));
 
 	if (!bonobo_ui_init_full ("test-ui", VERSION, &argc, argv,
@@ -286,25 +287,25 @@ main (int argc, char **argv)
 		GtkWidget *path_entry, *state_entry;
 
 		button = gtk_button_new_with_label ("Press me to test!");
-		gtk_signal_connect (GTK_OBJECT (button), "clicked",
+		g_signal_connect (GTK_OBJECT (button), "clicked",
 				    (GtkSignalFunc) cb_do_quit, NULL);
 		gtk_widget_show (GTK_WIDGET (button));
 		gtk_box_pack_start_defaults (GTK_BOX (box), button);
 
 		button = gtk_button_new_with_label ("Dump Xml tree");
-		gtk_signal_connect (GTK_OBJECT (button), "clicked",
+		g_signal_connect (GTK_OBJECT (button), "clicked",
 				    (GtkSignalFunc) cb_do_dump, win);
 		gtk_widget_show (GTK_WIDGET (button));
 		gtk_box_pack_start_defaults (GTK_BOX (box), button);
 
 		button = gtk_button_new_with_label ("Popup");
-		gtk_signal_connect (GTK_OBJECT (button), "clicked",
+		g_signal_connect (GTK_OBJECT (button), "clicked",
 				    (GtkSignalFunc) cb_do_popup, win);
 		gtk_widget_show (GTK_WIDGET (button));
 		gtk_box_pack_start_defaults (GTK_BOX (box), button);
 
 		button = gtk_button_new_with_label ("Hide toolbar");
-		gtk_signal_connect (GTK_OBJECT (button), "clicked",
+		g_signal_connect (GTK_OBJECT (button), "clicked",
 				    (GtkSignalFunc) cb_do_hide_toolbar, win);
 		gtk_widget_show (GTK_WIDGET (button));
 		gtk_box_pack_start_defaults (GTK_BOX (box), button);
@@ -316,7 +317,7 @@ main (int argc, char **argv)
 
 		state_entry = gtk_entry_new ();
 		gtk_entry_set_text (GTK_ENTRY (state_entry), "1");
-		gtk_signal_connect (GTK_OBJECT (state_entry), "changed",
+		g_signal_connect (GTK_OBJECT (state_entry), "changed",
 				    (GtkSignalFunc) cb_set_state, path_entry);
 		gtk_widget_show (GTK_WIDGET (state_entry));
 		gtk_box_pack_start_defaults (GTK_BOX (box), state_entry);
@@ -325,8 +326,8 @@ main (int argc, char **argv)
 		bonobo_window_set_contents (win, box);
 	}
 
-	gtk_signal_connect (GTK_OBJECT (win), "size_request", 
-			    GTK_SIGNAL_FUNC (slow_size_request), NULL);
+	g_signal_connect (GTK_OBJECT (win), "size_request", 
+			    G_CALLBACK (slow_size_request), NULL);
 
 	componenta = bonobo_ui_component_new ("A");
 	bonobo_object_unref (BONOBO_OBJECT (componenta));
@@ -335,21 +336,18 @@ main (int argc, char **argv)
 	componentb = bonobo_ui_component_new ("B");
 	componentc = bonobo_ui_component_new ("C");
 
+
 	bonobo_ui_component_set_container (componenta, corba_container, NULL);
 	bonobo_ui_component_set_container (componentb, corba_container, NULL);
 	bonobo_ui_component_set_container (componentc, corba_container, NULL);
 
 	global_component = componenta;
 
-	ev = &real_ev;
-	CORBA_exception_init (ev);
-
 	fname = bonobo_ui_util_get_ui_fname (NULL, "../doc/std-ui.xml");
 	if (fname && g_file_test (fname, G_FILE_TEST_EXISTS)) {
 		fprintf (stderr, "\n\n--- Add std-ui.xml ---\n\n\n");
 		bonobo_ui_util_set_ui (componenta, NULL, "../doc/std-ui.xml",
 				       "gnomecal", NULL);
-		bonobo_ui_component_thaw (componenta, NULL);
 
 /*		bonobo_ui_component_set_prop (
 			componenta, "/menu/Preferences",
@@ -364,6 +362,7 @@ main (int argc, char **argv)
 	}
 	g_free (fname);
 
+
 	bonobo_ui_component_freeze (componenta, NULL);
 
 	fprintf (stderr, "\n\n--- Remove A ---\n\n\n");
@@ -391,21 +390,24 @@ main (int argc, char **argv)
 		BonoboControl *control = bonobo_control_new (widget);
 		BonoboUIComponent *componentp;
 
-		gtk_signal_connect (GTK_OBJECT (widget), "button_press_event",
+		g_signal_connect (GTK_OBJECT (widget), "button_press_event",
 				    G_CALLBACK (do_sane_popup), control);
 		componentp = bonobo_control_get_popup_ui_component (control);
+#if 1
 		bonobo_ui_component_set (componentp, "/", "<popups>"
 					 "<popup name=\"button3\"/></popups>", ev);
 		g_assert (!BONOBO_EX (ev));
 		bonobo_ui_component_set_translate (
 			componentp, "/popups/button3", simpleb, ev);
 		g_assert (!BONOBO_EX (ev));
-		
+#endif
+
 		gtk_widget_show (widget);
 		bonobo_ui_component_object_set (componenta,
 						"/menu/File/MyControl",
 						BONOBO_OBJREF (control),
 						ev);
+		bonobo_object_unref (BONOBO_OBJECT (control));
 		g_assert (!BONOBO_EX (ev));
 	}
 
@@ -419,6 +421,7 @@ main (int argc, char **argv)
 						"/Toolbar/AControl",
 						BONOBO_OBJREF (control),
 						ev);
+		bonobo_object_unref (BONOBO_OBJECT (control));
 		g_assert (!BONOBO_EX (ev));
 	}
 
@@ -517,8 +520,9 @@ main (int argc, char **argv)
 						NULL);
 
 		id = gtk_timeout_add (100, (GSourceFunc) update_progress, widget);
-		gtk_signal_connect (GTK_OBJECT (widget), "destroy",
-				    GTK_SIGNAL_FUNC (disconnect_progress), GUINT_TO_POINTER (id));
+		g_signal_connect (GTK_OBJECT (widget), "destroy",
+				    G_CALLBACK (disconnect_progress), GUINT_TO_POINTER (id));
+		bonobo_object_unref (BONOBO_OBJECT (control));
 	}
 
 	bonobo_ui_component_set_status (componenta, "This is a very long status message "
@@ -533,7 +537,7 @@ main (int argc, char **argv)
 
 	g_warning ("Begginning stress test, this may take some time ...");
 	for (i = 0; i < 100; i++) {
-		bonobo_ui_component_freeze (componenta, ev);
+		bonobo_ui_component_freeze (componentc, ev);
 		g_assert (!BONOBO_EX (ev));
 		
 		bonobo_ui_component_set_translate (componentc, "/commands",
@@ -546,7 +550,7 @@ main (int argc, char **argv)
 		bonobo_ui_component_set_translate (componentc, "/menu/File", simpled, ev);
 		g_assert (!BONOBO_EX (ev));
 
-		bonobo_ui_component_thaw (componenta, ev);
+		bonobo_ui_component_thaw (componentc, ev);
 		g_assert (!BONOBO_EX (ev));
 	}
 	g_warning ("Done stress test");
@@ -594,11 +598,9 @@ main (int argc, char **argv)
 	bonobo_object_unref (BONOBO_OBJECT (componentb));
 	bonobo_object_unref (BONOBO_OBJECT (componentc));
 
-	bonobo_object_unref (BONOBO_OBJECT (container));
 	gtk_widget_destroy (GTK_WIDGET (win));
 
 	CORBA_exception_free (ev);
 
-	return 0;
+	return bonobo_ui_debug_shutdown ();
 }
-
